js_host="127.0.0.1"
js_port=9021

-- this implements a control daemon that implements
-- a configurable interface to manage joystick input
-- and map it to strings

-- this will hold the currently active button map
-- initially, we set some default values:
button_map = {
	["0"] = "\n",
	["1"] = " "
}

-- this will hold actions for simple direction movements of certain axes
axis_map = {}

-- this will hold static info about the axes
axis_info={}

-- this indicates whether the daemon is going to quit
quit = 0

socket = require("socket")
s = assert(socket.bind(js_host, js_port))
i, p = s:getsockname()
assert(i, p)
while quit == 0 do
	c = assert(s:accept())
	-- print("CONNECT", c:getpeername())
	l, e = c:receive()
	while not e do
		--   comment following line in to see some debug output...
		-- io.stderr:write(l.."\n")
		_, _, cmd = string.find(l, "^(%w+)")
		if cmd then
			-- print(l)
		end
		if cmd == "buttonpress" then
			_, _, btn = string.find(l, "^buttonpress (%d+)$")
			if button_map[btn] then
				c:send(button_map[btn])
			end
			c:shutdown()
		elseif cmd == "axismotion" then
			_, _, axis, axisabs = string.find(l, "^axismotion (-?%d+) (-?%d+)$")
			axis = tonumber(axis)
			axisabs = tonumber(axisabs)
			if not axis_info[axis] then
				axis_info[axis]={
					low=32768,
					high=-32768,
					trighigh=0,
					triglow=0
					}
			end
			if not axis_map[axis] then
				axis_map[axis] = {}
			end
			if axis_info[axis].low > axisabs then
				axis_info[axis].low = axisabs
			end
			if axis_info[axis].high < axisabs then
				axis_info[axis].high = axisabs
			end
			if axis_info[axis].trighigh == 1 and axisabs < (axis_info[axis].high * 7 / 10) then
				axis_info[axis].trighigh = 0
			end
			if axis_info[axis].triglow == 1 and axisabs > (axis_info[axis].low * 3 / 10) then
				axis_info[axis].triglow = 0
			end
			if axis_info[axis].trighigh == 0 and axisabs > (axis_info[axis].high * 8 / 10) then
				axis_info[axis].trighigh = 1
				if axis_map[axis]["+"] then
					c:send(axis_map[axis]["+"])
				end
			elseif axis_info[axis].triglow == 0 and axisabs < (axis_info[axis].low * 2 / 10) then
				axis_info[axis].triglow = 1
				if axis_map[axis]["-"] then
					c:send(axis_map[axis]["-"])
				end
			end
			c:shutdown()
		elseif cmd == "mapbutton" then
			_, _, btn, mapstr = string.find(l, "^mapbutton (%d+) (.*)$")
			mapstr = string.gsub(mapstr, "\\n", "\n")
			mapstr = string.gsub(mapstr, "\\e", "\27")
			mapstr = string.gsub(mapstr, "\\s", " ")
			mapstr = string.gsub(mapstr, "\\t", "\t")
			mapstr = string.gsub(mapstr, "\\\\", "\\")
			button_map[btn] = mapstr
		elseif cmd == "mapaxis" then
			_, _, axis, direction, mapstr = string.find(l, "^mapaxis (%d+) (.) (.*)$")
			axis = tonumber(axis)
			mapstr = string.gsub(mapstr, "\\n", "\n")
			mapstr = string.gsub(mapstr, "\\e", "\27")
			mapstr = string.gsub(mapstr, "\\s", " ")
			mapstr = string.gsub(mapstr, "\\t", "\t")
			mapstr = string.gsub(mapstr, "\\\\", "\\")
			if not axis_map[axis] then
				axis_map[axis] = {}
			end
			axis_map[axis][direction] = mapstr
		elseif cmd == "unmapbuttons" then
			button_map = {}
		elseif cmd == "unmapaxes" then
			axis_map = {}
		elseif cmd == "quit" then
			c:shutdown()
			quit = 1
		elseif cmd then
			-- io.stderr:write("Got unknown command: "..cmd.."\n")
		end
		l, e = c:receive()
	end
	c:close()
end
os.exit(0)
